import datetime, pytz

from django.core.exceptions import ValidationError
from django.forms.fields import MultiValueField, DateField, TimeField
from django.utils.translation import ugettext as _
from django.core.validators import EMPTY_VALUES

from timezone_field.forms import TimeZoneFormField
from .widgets import SplitTimeTimeZoneWidget, SplitHiddenTimeTimeZoneWidget, \
    SplitDateTimeTimeZoneWidget, SplitHiddenDateTimeTimeZoneWidget, \
    AdminSplitDateTimeTimeZone, AdminSplitTimeTimeZone


__all = (
    'SplitTimeTimeZoneField', 'AdminSplitTimeTimeZoneField',
    'SplitDateTimeTimeZoneField', 'AdminSplitDateTimeTimeZoneField'
)


class SplitTimeTimeZoneField(MultiValueField):
    widget = SplitTimeTimeZoneWidget
    hidden_widget = SplitHiddenTimeTimeZoneWidget
    empty_values = list(EMPTY_VALUES)

    default_error_messages = {
        'invalid_time': _('Enter a valid time.'),
        'invalid_time_zone': _('Enter a valid time zone.'),
    }

    def __init__(self, input_time_formats=None, *args, **kwargs):
        errors = self.default_error_messages.copy()
        if 'error_messages' in kwargs:
            errors.update(kwargs['error_messages'])
        localize = kwargs.get('localize', False)
        fields = (
            TimeField(input_formats=input_time_formats,
                      error_messages={'invalid': errors['invalid_time']},
                      localize=localize),
            TimeZoneFormField(error_messages={'invalid': errors['invalid_time']},
                      localize=localize, initial="Australia/Sydney"),

        )
        super(SplitTimeTimeZoneField, self).__init__(fields, *args, **kwargs)

    def compress(self, data_list):
        if data_list:
            if data_list[0] in self.empty_values
                return None
            # Raise a validation error if time or date is empty
            # (possible if SplitTimeTimeZoneField has required=False).
            if data_list[0] in self.empty_values:
                raise ValidationError(self.error_messages['invalid_time'], code='invalid_time')
            if data_list[1] in self.empty_values:
                raise ValidationError(self.error_messages['invalid_time_zone'], code='invalid_time_zone')
            result = data_list[0].replace(tzinfo=data_list[1])
            return result
        return None


class SplitDateTimeTimeZoneField(MultiValueField):
    widget = SplitDateTimeTimeZoneWidget
    hidden_widget = SplitHiddenDateTimeTimeZoneWidget
    empty_values = list(EMPTY_VALUES)

    default_error_messages = {
        'invalid_date': _('Enter a valid date.'),
        'invalid_time': _('Enter a valid time.'),
        'invalid_time_zone': _('Enter a valid time zone.'),
    }

    def __init__(self, input_date_formats=None, input_time_formats=None, *args, **kwargs):
        errors = self.default_error_messages.copy()
        if 'error_messages' in kwargs:
            errors.update(kwargs['error_messages'])
        localize = kwargs.get('localize', False)
        fields = (
            DateField(input_formats=input_date_formats,
                      error_messages={'invalid': errors['invalid_date']},
                      localize=localize),
            TimeField(input_formats=input_time_formats,
                      error_messages={'invalid': errors['invalid_time']},
                      localize=localize),
            TimeZoneFormField(error_messages={'invalid': errors['invalid_time']},
                      localize=localize, initial="Australia/Sydney"),

        )
        super(SplitDateTimeTimeZoneField, self).__init__(fields, *args, **kwargs)

    def compress(self, data_list):
        if data_list:
            if data_list[0] in self.empty_values and data_list[1] in self.empty_values
                return None
            # Raise a validation error if time or date is empty
            # (possible if SplitDateTimeTimeZoneField has required=False).
            if data_list[0] in self.empty_values:
                raise ValidationError(self.error_messages['invalid_date'], code='invalid_date')
            if data_list[1] in self.empty_values:
                raise ValidationError(self.error_messages['invalid_time'], code='invalid_time')
            if data_list[2] in self.empty_values:
                raise ValidationError(self.error_messages['invalid_time_zone'], code='invalid_time_zone')
            result = datetime.datetime.combine(*data_list[0:2]).replace(tzinfo=data_list[2])
            return result
        return None




from django.conf import settings as _settings

from .widgets import *
from .fields import *

__version__ = '0.1.4'

if getattr(_settings, 'USE_TZ_FIELDS', False):
    from django.contrib.admin import options
    from django.db import models

    options.FORMFIELD_FOR_DBFIELD_DEFAULTS.update({
        models.DateTimeField: {
          'form_class': SplitDateTimeTimeZoneField,
          'widget': AdminSplitDateTimeTimeZone,
        },
        models.TimeField: {
          'form_class': SplitTimeTimeZoneField,
          'widget': AdminSplitTimeTimeZone,
        }
    })



import pytz
import datetime
import decimal

from django.forms.widgets import Select, MultiWidget, DateInput, TimeInput
from django.contrib.admin.widgets import AdminDateWidget, AdminTimeWidget
from django.utils.html import format_html
from django.utils.translation import ugettext as _
from django.utils import timezone
from django.conf import settings


__all__ = (
    'TimeZoneSelect', 'SplitDateTimeTimeZoneWidget', 
    'SplitHiddenDateTimeTimeZoneWidget', 'AdminSplitDateTimeTimeZone'
)


def _utcoffset(tz):
    """
    Returns the UTC Offset measured in hours for a timezone for this time of
    the year.
    """
    tz = pytz.timezone(tz)
    offset = datetime.datetime.now(tz).utcoffset()
    hours = (offset.days * 86400.0 + offset.seconds) / 3600.0
    if hours == int(hours):
        hours = int(hours)
    if hours >= 0:
        return "+" + str(hours)
    return str(hours)
    

class TimeZoneSelect(Select):
    """
    A Widget that allows the selection of a timezone.
    """
    def __init__(self, attrs=None):
        super(TimeZoneSelect, self).__init__(attrs)
        now = timezone.now()
        self.choices = [(tz, "%s (%s)" % (tz, _utcoffset(tz)))
            for tz in pytz.common_timezones]


class SplitTimeTimeZoneWidget(MultiWidget):
    """
    A Widget that splits time & timezone input into one <input type="text"> 
    boxes plus a timezone select.
    """
    def __init__(self, attrs=None, time_format=None):
        widgets = (TimeInput(attrs=attrs, format=time_format),
                   TimeZoneSelect(attrs=attrs))
        super(SplitTimeTimeZoneWidget, self).__init__(widgets, attrs)

    def decompress(self, value):
        tzinfo = timezone.get_current_timezone()
        if value:
            if value.tzinfo:
                tzinfo = value.tzinfo
            return [value.replace(microsecond=0), tzinfo]
        return [None, tzinfo]


class SplitHiddenTimeTimeZoneWidget(SplitTimeTimeZoneWidget):
    """
    A Widget that splits timezone input into two <input type="hidden"> inputs.
    """
    is_hidden = True

    def __init__(self, attrs=None, time_format=None):
        super(SplitHiddenTimeTimeZoneWidget, self).__init__(attrs, time_format)
        for widget in self.widgets:
            widget.input_type = 'hidden'
            widget.is_hidden = True


class AdminSplitTimeTimeZone(SplitTimeTimeZoneWidget):
    """
    A SplitTimeTimeZoneWidget Widget that has some admin-specific styling.
    """
    def __init__(self, attrs=None):
        widgets = [AdminTimeWidget, TimeZoneSelect]
        # Note that we're calling MultiWidget, not SplitTimeTimeZoneWidget, 
        # because we want to define widgets.
        MultiWidget.__init__(self, widgets, attrs)

    def format_output(self, rendered_widgets):

        return format_html('<p class="datetime">{0} {1} {2}</p>',
                           _('Time:'), rendered_widgets[0],
                           rendered_widgets[1])


class SplitDateTimeTimeZoneWidget(MultiWidget):
    """
    A Widget that splits datetime & timezone input into two <input type="text"> 
    boxes plus a timezone select.
    """
    def __init__(self, attrs=None, date_format=None, time_format=None):
        widgets = (DateInput(attrs=attrs, format=date_format),
                   TimeInput(attrs=attrs, format=time_format),
                   TimeZoneSelect(attrs=attrs))
        super(SplitDateTimeTimeZoneWidget, self).__init__(widgets, attrs)

    def decompress(self, value):
        tzinfo = timezone.get_current_timezone()
        if value:
            if value.tzinfo:
                tzinfo = value.tzinfo
            return [value.date(), value.time().replace(microsecond=0), tzinfo]
        return [None, None, tzinfo]


class SplitHiddenDateTimeTimeZoneWidget(SplitDateTimeTimeZoneWidget):
    """
    A Widget that splits datetimezone input into three <input type="hidden"> inputs.
    """
    is_hidden = True

    def __init__(self, attrs=None, date_format=None, time_format=None):
        super(SplitHiddenDateTimeTimeZoneWidget, self).__init__(attrs, date_format, time_format)
        for widget in self.widgets:
            widget.input_type = 'hidden'
            widget.is_hidden = True


class AdminSplitDateTimeTimeZone(SplitDateTimeTimeZoneWidget):
    """
    A SplitDateTimeTimeZoneWidget Widget that has some admin-specific styling.
    """
    def __init__(self, attrs=None):
        widgets = [AdminDateWidget, AdminTimeWidget, 
          TimeZoneSelect]
        # Note that we're calling MultiWidget, not SplitDateTimeTimeZoneWidget, 
        # because we want to define widgets.
        MultiWidget.__init__(self, widgets, attrs)

    def format_output(self, rendered_widgets):

        return format_html('<p class="datetime">{0} {1}<br />{2} {3} {4}</p>',
                           _('Date:'), rendered_widgets[0],
                           _('Time:'), rendered_widgets[1],
                           rendered_widgets[2])

